/***********************************************************************

    Power Scheduler Control System

	GenBoost1.C : Synchronising "Generator Booster" Inverter Algorithm

    (c) Equipmake 2005

______________________________________________________________________________________________
    WHAT    |    WHEN   |      WHO      | WHY
----------------------------------------------------------------------------------------------
    0.1     | 03/02/05  | I.Foley       | Started.
----------------------------------------------------------------------------------------------
	0.2		| 15/01/06	| I. Foley		| Additional Input Current Sensor Added.
			|			|				| Precharge Bypass Relay moved from RE0 to RC2
-----------------------------------------------------------------------------------------------
	0.3		| 16/2/06	| I. Folay		| Seems to waork again after big blow up
----------------------------------------------------------------------------------------------
	0.4		|			|				| Toggle Mains facility in
//**********************************************************************************************/
// This is to send the version number so the interface program can show
// on the laptop which version of code is in each processor
// Version 1.0 will be CodeVersionMain 1 CodeVersionSub 0 etc
#define CodeVersionMain 0
#define CodeVersionSub 2

#define DECLARE_GLOBALS 1
//#include "C:\Equipmake\ProductionInverters\GenBoost1\globalGenboost1.h"	// Include globals and declare variables
//#include "18f458.h"  
//#include "htc.h"
#include "GlobalGenboost1.h"                                                      // Include globals and declare variables
#include <p18f458.h>
//#include <timers.h>
//#include <I2C.h>
//#include <capture.h>
//#include <usart.h>
//#include <adc.h>

#undef DECLARE_GLOBALS  

const rom Sine[] = {512,544,576,608,639,670,700,730,759,786,813,838,862,885,907,926,944,961,975,988,999,1008,1015,1020,1023,1023,
				   1023,1020,1015,1008,999,988,975,961,944,926,907,885,862,838,813,786,759,730,700,670,639,608,576,544,512,
				   480,448,416,385,354,324,294,265,238,211,186,162,139,117,98,80,63,49,36,25,16,9,4,1,1,1,
				   4,9,16,25,36,49,63,80,98,117,139,162,186,211,238,265,294,324,354,385,416,448,480};

/**********************************************************************
	IStat values
**********************************************************************/
#define I_BOOT			 			0
#define I_SWITCHED_OFF	 			1
#define I_STARTING		 			2
#define I_RUNNING		 			3
/*********************************************************************
	ADC Channel Definitions
**********************************************************************/
#define IMON						0
#define GMON						1
#define VMON						2
#define BMON						3
#define BIMON						4
/*********************************************************************
	Phase Definitions
*********************************************************************/
#define	UNKNOWN						0
#define BELOWZERO					1	
#define	ABOVEZERO					2
#define CROSSINGZERO				3
/*********************************************************************
	Mains Input Stage Definitions
*********************************************************************/
#define	MAINS_OFF					0
#define SYNCHRONIZE_MAINS			1
#define	CONNECT_MAINS_CONTACTOR		2
#define MAINS_RUNNING				3
#define	DISCONNECT_MAINS_CONTACTOR	4
/***********************************************************************
	Parameters
************************************************************************/
#define TICK						1			// Iteration rate scaler (10mS)
#define BOOT_TIME					100/TICK	// Boot up time to allow ADC settle and CAN comms
#define MIN_GENERATOR_ON_VOLTS		600			// Min Voltage to make decsion generator is ON
#define MAX_GENERATOR_OFF_VOLTS		550			// Volts at which to decide generator has lost volts
#define	MAX_CHARGING_CURRENT		60
#define SETPOINT					775			// Target Voltage (Should be 775)
#define INTERNAL_DC_OFFSET			0
#define EXTERNAL_DC_OFFSET			0
#define GENERATOR_VOLTS_OFFSET 		0
#define GENERATORVOLTSOFFSET		517
#define OUTPUTVOLTSOFFSET			512
#define MAX_BATTERY_VOLTS			2800
#define GENHYSTERESISLEVEL			250	
#define OUTPUTVOLTSHYSTERESISLEVEL	250
#define MAINSCONTACTORONDELAY		5000		// Delay before mains contactor on after synchronisation
#define DESYNCTIME					5000
#define MAINSONDELAYTIME			5000
/***********************************************************************
	Variables
************************************************************************/
union Timers timer;
union VBResult VBunion;
union ADCResult ADCres;
union CANResult Result;
UCHAR	IStat					= I_BOOT;
WORD	BootTimer				= BOOT_TIME;
WORD Watchdog_Counter			= 0;
UCHAR   TmpString[20]			="                   ";	// Used as buffer in VBComms
UCHAR	TmpMessage[10];
UCHAR 	length					= 0;			// Used in VBComms
BOOL	FirstPass				= FALSE;		// Used in VBComms
UCHAR	LineNum					= 1;			// Used in VBComms
SCHAR	SineCounter				= 0;
SCHAR	AverageSineCounter		= 0;
SCHAR	MainsSineCounter		= 0;
int		OutputVolts				= 0;
int		Offset					= 512;
int		DCOffset				= 0;
int		MaxOutputVolts			= 0;
unsigned int    ServoOutput		= 0;
long    templong				= 0;
int		tempint					= 0;
int		MaxBatteryCurrent		= 0;
int		MaxBatteryCurrentP		= 0;
int		SoftStart				= 0;
int		Stop					= 0;
BOOL	FirstPassInverter		= TRUE;
int		OutputCurrent			= 0;
int		BatteryVolts			= 0;
int		GeneratorVolts			= 0;
int		BatteryCurrent			= 0;
int		MaxGeneratorVolts		= 0;
int		MaxGeneratorVoltsP		= 0;
int		TempGenVolts			= 0;
UCHAR	Output					= 0;
UCHAR	tmpchar					= 0;
UCHAR	k						= 0;
UCHAR	i						= 0;
INT32	tmplong					= 0;
int		test					= 0;
int		Maxtst					= 0;
SCHAR	temp					= 0;
BOOL	ChargeBatteries			= FALSE;
UCHAR	DemandVolts				= 0;
int		IntDemandVolts			= 0;
int		IntOffset				= 512;
int		OutputScale				= 0;
int		ServoInput				= 0;
int		MaxOutputCurrent		= 0;
int		MaxOutputCurrentP		= 0;
int		ADCTimer				= 50;
UCHAR	NextADCChan				= 0;
int		MaxOutputVoltsP			= 0;
UCHAR	ErrorIntegrator			= 1;
int		ErrorGain				= 0;
int		Error					= 0;
BOOL	Tripped					= FALSE;
WORD	PrimaryOvercurrentCounter = 0;
WORD	PrimaryOvercurrentTimer  = 0;
WORD	OvercurrentResetTimer	= 0;
UCHAR	Pcounter				= 0;
BOOL	External_Mains_On		= FALSE;
BOOL	External_Mains_On_P		= FALSE;
BOOL	System_On				= FALSE;
UCHAR	Led1					= OFF;
UCHAR	Led2					= OFF;
UCHAR	Led3					= OFF;
UCHAR	Led4					= OFF;
UCHAR	Led5					= OFF;
UCHAR	VBCommsCounter			= 0;
UCHAR	SyncByteCounter			= 0;
UCHAR	convnum					= 0;
UCHAR	AsciiHex				= 0;
UCHAR	ChrLength				= 0;
UCHAR	Channel					= 0;
UCHAR	Mains_Contactor			= 0;
int		MaxOutputScale			= 0;
int		MaxBatteryVolts			= 0;
int		MaxBatteryVoltsP		= 0;
int 	AverageGeneratorVolts	= 0;
int 	AverageGeneratorVoltsP	= 0;
int		AverageOutputVolts		= 0;
int		AverageOutputVoltsP		= 0;
int		PWMMax					= 0;
int		PWMMaxP					= 0;
BOOL	PWMLim					= FALSE;
int     icounter				= 0;
int		MainsInputCurrent		= 0;
int		MaxMainsInputCurrent	= 0;
int		MaxMainsInputCurrentP	= 0;
int		LoadCurrentPhase		= 0;
int		Time					= 0;
BOOL	PhaseTrigger			= 0;
BOOL	JustReadChan3			= FALSE;
UCHAR	OutputVoltsPhase		= UNKNOWN;
UCHAR	GenVoltsPhase			= UNKNOWN;
int 	VoltagePhase			= 0;
UCHAR	MainsInputStatus		= MAINS_OFF;
int		ContactorDelayTimer		= MAINSCONTACTORONDELAY;
UCHAR	CANOutputCounter		= 0;
BOOL	GenReverseFeed			= FALSE;
BOOL	InverterAvailable		= TRUE;
BOOL	Generator				= OFF;
UCHAR	GeneratorStatus			= 0;
WORD	DeSyncCounter			= 0;
BOOL	DeSyncing				= FALSE;
WORD	MainsOnDelayTimer		= MAINSONDELAYTIME;
BOOL	ToggleMains				= FALSE;
BOOL	ToggleMainsFlag			= FALSE;

void OverCurrentTrip(void);


#pragma code lowVector=0x18
void atLowVector(void)
{
	_asm GOTO OverCurrentTrip _endasm
}
#pragma code

#pragma interruptlow OverCurrentTrip
void OverCurrentTrip(void)
{
	
	PrimaryOvercurrentCounter++;
	if (PrimaryOvercurrentCounter > 2000)
	{
		ECCP1CON = 0b00000000;	// Switch off PWM mode
		PORTDbits.RD4 = 0;		// Switch FET drives to off
		PORTDbits.RD5 = 0;
		SoftStart = 0;
		FirstPassInverter = 1;			
		tmpchar = PORTB;
		Tripped = 1;
	}
	INTCON3bits.INT1IF = 0;

	Nop();
	Nop();
}




void GENBOOST1_Initialise (void)
{
	long whiletimer;
	
	//*************Set up IO Digital port bits for Input or Output********************************

	// Port A
	TRISAbits.TRISA0 = 1;	//Analogue In Output Current Monitor
	TRISAbits.TRISA1 = 1;	//Analogue In Generator Voltage Monitor
	TRISAbits.TRISA2 = 1;	//Analogue In Output Voltage Monitor
	TRISAbits.TRISA3 = 1;	//Analogue In Battery Voltage Monitor
	TRISAbits.TRISA4 = 1;	//Not Used
	TRISAbits.TRISA5 = 1;	//Analogue In Battery Current Monitor
	TRISAbits.TRISA6 = 1;	//Oscillator
	// Port B
	TRISBbits.TRISB0 = 1;	//Overcurrent Interrupt Trip
	TRISBbits.TRISB1 = 0;	//Overcurrent Trip Reset '1' = RESET '0' = LATCH OFF
	PORTBbits.RB1 = 1;		// Set Overcurrent Trip reset to 1
	TRISBbits.TRISB2 = 0;	//CAN TX
	TRISBbits.TRISB3 = 1;	//CAN RX
	TRISBbits.TRISB4 = 0;	//Debugging Output
	TRISBbits.TRISB5 = 1;	//Not Used
	// B6 & B7 are reserved for the ICD
	// Port C
	TRISCbits.TRISC0 = 0;	//SCL Aux
	TRISCbits.TRISC1 = 0;	//SDA Aux
	TRISCbits.TRISC2 = 0;	//Precharge Bypass Relay (was called Inrush Limit Relay)
	TRISCbits.TRISC3 = 0;	//Inverter Running Flag from Inverter to Scheduler
	TRISCbits.TRISC4 = 1;	//SDI
	TRISCbits.TRISC5 = 0;	//SDO
	TRISCbits.TRISC6 = 0;	//RS 232 TX
	TRISCbits.TRISC7 = 1;	//RS 232 RX
	// Port D
	CMCON = 0x07;
	TRISDbits.TRISD0 = 0;	//LED1
	TRISDbits.TRISD1 = 0;	//LED2
	TRISDbits.TRISD2 = 0;	//LED3
	TRISDbits.TRISD3 = 0;	//LED4
	TRISDbits.TRISD4 = 1;	//PWM Output
	TRISDbits.TRISD5 = 1;	//PWM Output
	TRISDbits.TRISD6 = 0;	//LED5
	TRISDbits.TRISD7 = 1;	//ON/OFF Switch
	// Port E
	PORTE = 0;
//	TRISEbits.TRISE0 = 0;	//Input Current Monitor
//	TRISEbits.TRISE1 = 0;	//RELAY2 External Mains Contactor
//	TRISEbits.TRISE2 = 0;	//RELAY3 Battery Disconnect Contactor
	TRISE = 0b00000001;
	
	PORTCbits.RC2 = 0;
	PORTEbits.RE1 = 0;
	PORTEbits.RE2 = 0;
	//***************End of Digital IO port allocation***********************************************

	// ********************Initialise USART to 19200 baud rate with 32MHz clock*****************

	OpenUSART(USART_TX_INT_OFF&USART_RX_INT_OFF&USART_ASYNCH_MODE&USART_EIGHT_BIT&USART_CONT_RX&USART_BRGH_HIGH,106);
	
		/************************CAN Address Assignments********************************************
	Channel						Address			Sent By
								
	Inverter Current High Byte	100H    byte 0	Inverter Processor	
	Inverter Current Low Byte	100H	byte 1	Inverter Processor
	Inverter Voltage High Byte	100H	byte 2	Inverter Processor
	Inverter Voltage Low Byte	100H	byte 3	Inverter Processor
	Battery Voltage High Byte	100H	byte 4  Inverter Processor
	Battery Voltage Low Byte	100H    byte 5  Inverter Processor
	Battery Current High Byte	100H    byte 6  Inverter Processor
	Battery Current Low Byte	100H    byte 7  Inverter Processor
	Generator Voltage High Byte	101H	byte 0	Gen Switch Box Processor
	Generator Voltage Low Byte	101H	byte 1  Gen Switch Box Processor
	Generator Current High Byte	101H	byte 2	Gen Switch Box Processor
	Generator Current Low Byte	101H	byte 3	Gen Switch Box Processor
	GenReverseFeedFlag			101H	byte 4	Gen Switch Box Processor	
								101H	byte 5	
								101H	byte 6  
								101H	byte 7  
	CloseContactorFlag			300H	byte 0 Inverter Processor	
								300H	byte 1  
								300H    byte 2	
								300H	byte 3
								300H	byte 4
								300H	byte 5	
		Devices Status (to 5SW)	400H	byte 0	Scheduler Processor	
		Devices Status(to GenSw)400H	byte 1	Scheduler Processor	
								400H	byte 2	
			 					400H	byte 3	
								400H	byte 4	
								400H	byte 5	
								400H	byte 6	
								401H	byte 0  
								401H	byte 1  
								401H	byte 2	
								401H	byte 3	
								401H	byte 4
								401H	byte 5	
				
	*******************************************************************************************/

	//************************ Set Up CAN Interface*********************************************
	// Set CAN module to configuration mode
	 CANCON = 0b10000000;
	// wait until config mode confirmed
    whiletimer = 16000000; // set timer to about 2 seconds before timeout for no CAN module response
	 while ((CANSTAT & 0b11100000) != (0b10000000))
	  {
	  whiletimer = whiletimer - 1;
	  if (whiletimer <= 0)
	  	break;
	 }

	// Set Up Baud Rate registers for CAN Baud Rate 1Mbit/s with a 32MHz clock rate (8MHz * 4)
	 BRGCON1 = 0b01000000;
	 BRGCON2 = 0b10101010;
	 BRGCON3 = 0b00000101;
	// Set Up Receive Registers
	 RXB0CON = 0b00100000;
	 RXB1CON = 0b00100000;
	// Set Up Acceptance Masks and filters for Register 0
	 RXM0SIDH = 0b11111111; // Test all bits
	 RXM0SIDL = 0b11111111;
	 RXF0SIDH = 0b10000000; // Set Filter to accept message 400H from Main Processor
	 RXF0SIDL = 0b00000000; // Filter will only accept standard identifiers
	 RXF1SIDH = 0b10000000; // Set filter 1 to accept message 400H from Main Processor
	 RXF1SIDL = 0b00000000; // Filter will only accept standard identifiers

	 RXM1SIDH = 0b11111111;
	 RXM1SIDL = 0b11111111;
	 
	 RXF2SIDH = 0b00100000; //Set filter 1 to accept message 101H from  Gen Switch Box
	 RXF2SIDL = 0b00100000; //Filter will only accept standard identifiers
	 RXF3SIDH = 0b00000000;
	 RXF3SIDL = 0b00000000;
	 RXF4SIDH = 0b00000000;
	 RXF4SIDL = 0b00000000;
	 RXF5SIDH = 0b00000000;
	 RXF5SIDL = 0b00000000;

	//Now set to Normal Only Mode
	 CANCON = 0b00000000;
	// Wait until Listen Only mode confirmed
	 whiletimer = 16000000;
	 while ((CANSTAT & 0b11100000) != (0b00000000))
	 {
	  whiletimer = whiletimer - 1;
	  if (whiletimer <= 0)
	  	break;
	 }
 
	OpenTimer0(TIMER_INT_OFF & T0_16BIT & T0_SOURCE_INT & T0_PS_1_1);           
	// Consider how to handle cold and warm boots (power interruption or
	// watchdog reset)
	
	OpenADC(ADC_FOSC_64 & ADC_RIGHT_JUST & ADC_6ANA_0REF, ADC_CH2 & ADC_INT_OFF);

	//Use Timer1 to time Zero Crossings of input and output Mains current
	T1CON = 0b10110101;
	

//	CMCON = 0b00000111;  // Switch Comparators off (I think)

 
	// Set up PWM control 
//	PR2 = 110;           // started at 125
//	ECCP1CON = 0b10001100;
//	ECCP1DEL = 10;
//	T2CON = 0b00000101;
	//Wait for 1st complete PWM cycle
//	while(!PIR1bits.TMR2IF);
	TRISDbits.TRISD4 = 0;
	PORTDbits.RD4 = 0;
	TRISDbits.TRISD5 = 0;
	PORTDbits.RD5 = 0;	
	INTCON  = 0b00000000;
//	INTCON2 = 0b01000000;
//	INTCON3 = 0b00001000;
}


void GENBOOST1_ReadCAN(void)
{

// *****************Read CAN data from Inverter Processor*********************
		
	if (RXB0CONbits.RXFUL)  
		{
			GeneratorStatus   =	RXB0D1;
			InverterAvailable = RXB0D2;
			
			if (GeneratorStatus & 0b00000001) // Generator Switched of by Scheduler
				Generator  = ON;
			else
				Generator = OFF;

			if (!InverterAvailable)
				//Tripped = 1;

		  	Result.br[0] = RXB0D5;
			Result.br[1] = RXB0D4;
			
		  	Result.br[0] = RXB0D7;
	 	 	Result.br[1] = RXB0D6;
			
	 		RXB0CONbits.RXFUL = 0;	// Reset Buffer Full Flag
		}
		

	if (RXB1CONbits.RXFUL)
		{
			GenReverseFeed		= RXB1D4;
			GenReverseFeed = FALSE; // ************For Debugging Only
		
			RXB1CONbits.RXFUL 	= 0;
		}

}


void WriteCAN(void)
{

	TXB1SIDH = 0b01100000;		//300H Address for Inverter Parameters frame 2
	TXB1SIDL = 0b00000000;
	TXB1D0   = Mains_Contactor;
	  
	TXB1DLC  = 0b00000001;		// Send 1 Bytes
	
	TXB1CON  = 0b00001011;		// Send Message

	TXB0SIDH = 0b00100000;		//100H Address for Inverter Parameters
	TXB0SIDL = 0b00000000;		// lower bits for 100H
	Result.lr = MaxOutputCurrentP;
	TXB0D0   = Result.br[1];
	TXB0D1	 = Result.br[0];
	Result.lr = AverageOutputVoltsP;
	TXB0D2	 = Result.br[1];
	TXB0D3   = Result.br[0];
	Result.lr = MaxBatteryVoltsP;
	TXB0D4   = Result.br[1];
	TXB0D5	 = Result.br[0];
	Result.lr = MaxBatteryCurrentP;
	TXB0D6	 = Result.br[1];
	TXB0D7   = Result.br[0];
	  
	TXB0DLC  = 0b00001000;		// Send 8 Bytes
	
	TXB0CON  = 0b00001000;		// Send Message



}

void GENBOOST1_ReadGenVolts(void)
{
	ADCON0bits.GO = 1;
		ADCTimer = 50;
		while (ADCTimer > 2)
		{
			ADCTimer--;
			if (!ADCON0bits.GO)
			{
			 	ADCres.br[0] = ADRESL;  // Read ADRESL into the lower byte
  				ADCres.br[1] = ADRESH;  // Read ADRESH into the high byte
				GeneratorVolts = ADCres.lr;
				ADCTimer = 1;
			}
		}
		if (GeneratorVolts > MaxGeneratorVolts)
			MaxGeneratorVolts = GeneratorVolts;
		
		if (GeneratorVolts > GENERATORVOLTSOFFSET)  // Above Zero Crossing Line
		{
			tempint = GeneratorVolts - GENERATORVOLTSOFFSET;
			if (GenVoltsPhase == BELOWZERO)
			{
				WriteTimer1(0);
				GenVoltsPhase = CROSSINGZERO;
				  PORTBbits.RB4 = 1;
			}
			else
				GenVoltsPhase = ABOVEZERO;
		}
		else // Below Zero Crossing Line
		{
			tempint = GENERATORVOLTSOFFSET - GeneratorVolts;
			if (tempint > GENHYSTERESISLEVEL)
			{
				GenVoltsPhase = BELOWZERO;
				  PORTBbits.RB4 = 0;
			}
		}
		AverageGeneratorVolts = (AverageGeneratorVolts + tempint);

 	//	if (tempint < 30)
	//		if (PhaseTrigger)
	//			LoadCurrentPhase = ReadTimer1();
}

void GENBOOST1_ReadOutputVolts(void)
{
	ADCON0bits.GO = 1;
		ADCTimer = 50;
		while (ADCTimer > 2)
		{
			ADCTimer--;
			if (!ADCON0bits.GO)
			{
			 	ADCres.br[0] = ADRESL;  // Read ADRESL into the lower byte
  				ADCres.br[1] = ADRESH;  // Read ADRESH into the high byte
				OutputVolts = ADCres.lr;
				ADCTimer = 1;
			}
		}
		if (OutputVolts > MaxOutputVolts)
			MaxOutputVolts = OutputVolts;
 
	    if (OutputVolts > OUTPUTVOLTSOFFSET)
			tempint = OutputVolts - OUTPUTVOLTSOFFSET;
		else
			tempint = OUTPUTVOLTSOFFSET - OutputVolts;
			
		AverageOutputVolts = (AverageOutputVolts + tempint);
}

void GENBOOST1_ReadADC(void)
{
switch (NextADCChan)
{
	case 0:
		ADCON0bits.GO = 1;
		ADCTimer = 50;		
		while (ADCTimer > 2)
		{
		ADCTimer--;
			if (!ADCON0bits.GO)
			{
			 	ADCres.br[0] = ADRESL;  // Read ADRESL into the lower byte
  				ADCres.br[1] = ADRESH;  // Read ADRESH into the high byte
				OutputCurrent = ADCres.lr;
				ADCTimer = 1;
			}
		}
		if (OutputCurrent > MaxOutputCurrent)
			MaxOutputCurrent = OutputCurrent;

		
		NextADCChan = 2;
		
	break;

	case 1:
	//Reserved for Generator Volts but read every itteration in another routine
	break;

	case 2:
		ADCON0bits.GO = 1;
		ADCTimer = 50;
		while (ADCTimer > 2)
		{	
			ADCTimer--;
			if (!ADCON0bits.GO)
			{
				ADCres.br[0] = ADRESL;  // Read ADRESL into the lower byte
  				ADCres.br[1] = ADRESH;  // Read ADRESH into the high byte
				OutputVolts = ADCres.lr;
			ADCTimer = 1;
			}
		}
		if (OutputVolts > MaxOutputVolts)
			MaxOutputVolts = OutputVolts;
			
		if (OutputVolts > OUTPUTVOLTSOFFSET)
		{
			tempint = OutputVolts - OUTPUTVOLTSOFFSET;
			if (OutputVoltsPhase == BELOWZERO)
			{
				VoltagePhase = ReadTimer1();
				if (VoltagePhase > 10000)
					VoltagePhase = 20000 - VoltagePhase;
				OutputVoltsPhase = CROSSINGZERO;
			}
			else
				OutputVoltsPhase = ABOVEZERO;
		}
		else
		{
			tempint = OUTPUTVOLTSOFFSET - OutputVolts;
			if (tempint > OUTPUTVOLTSHYSTERESISLEVEL)
				{
					OutputVoltsPhase = BELOWZERO;	
				}
			
		}
		AverageOutputVolts = (AverageOutputVolts + tempint);
	
		if (JustReadChan3)
			NextADCChan = 4;
		else
			NextADCChan = 3;
	break;


	case 3:
		ADCON0bits.GO = 1;
		ADCTimer = 50;
		k=0;
		while (ADCTimer > 2)
		{
		 k++;
		 ADCTimer --;
			if (!ADCON0bits.GO)
			{
				ADCres.br[0] = ADRESL;  // Read ADRESL into the lower byte
  				ADCres.br[1] = ADRESH;  // Read ADRESH into the high byte
				BatteryVolts = ADCres.lr*4;
				ADCTimer = 1;
			}
		}
		
			if (BatteryVolts > MaxBatteryVolts)
				MaxBatteryVolts = BatteryVolts;
		JustReadChan3 = TRUE;
		NextADCChan= 5;
	break;
	

	case 4:
		ADCON0bits.GO = 1;
		ADCTimer = 50;
		while (ADCTimer > 2)
		{
			ADCTimer--;
			if (!ADCON0bits.GO)
			{
				ADCres.br[0] = ADRESL;  // Read ADRESL into the lower byte
  				ADCres.br[1] = ADRESH;  // Read ADRESH into the high byte
				BatteryCurrent = ADCres.lr - 512;
				ADCTimer = 1;
			}
		}

			if (BatteryCurrent > MaxBatteryCurrent)
				MaxBatteryCurrent = BatteryCurrent;
		 JustReadChan3 = FALSE;
		NextADCChan = 5;
	break;

	case 5:
		ADCON0bits.GO = 1;
		ADCTimer = 50;
		while (ADCTimer > 2)
		{
			ADCTimer--;
			if (!ADCON0bits.GO)
			{
				ADCres.br[0] = ADRESL;  // Read ADRESL into the lower byte
  				ADCres.br[1] = ADRESH;  // Read ADRESH into the high byte
				MainsInputCurrent = ADCres.lr;
				ADCTimer = 1;
			}
		}

		if (MainsInputCurrent > MaxMainsInputCurrent)
			MaxMainsInputCurrent = MainsInputCurrent;
		//This is to check the phase of input and output current
		if (MainsInputCurrent < 30)
		{
		//	WriteTimer1(0);
			PhaseTrigger = TRUE;
		}
		else
			PhaseTrigger = FALSE;

		NextADCChan = 0;
	break;

	
  }
	  
// SetChanADC(NextADCChan<<3);
   SetChanADC(1<<3);
}

unsigned char strlength(void)
{
char tmpchar;
 k = 0;
	while (TmpString[k] != 0)
	{
	  tmpchar = TmpString[k];
		k++;
	}
	return(k);
}

/****************************************************************************************/
/*  Run_Inverter																			*/
/*  This is the Main Inverter "Engine" or State Machine									*/
/****************************************************************************************/
void GENBOOST1_Run_Inverter (void)
{

	switch (IStat)
	{
		case I_BOOT :
			BootTimer--;
			if (BootTimer < 1)
				IStat = I_SWITCHED_OFF;
				
		break;					// Still booting up so do nothing

		case I_SWITCHED_OFF :

			ECCP1CON = 0b00000000;	// Switch off PWM mode
			PORTDbits.RD4 = 0;		// Switch FET drives to off
			PORTDbits.RD5 = 0;
			SoftStart = 0;
			PrimaryOvercurrentCounter = 0;
			PrimaryOvercurrentTimer = 0;
			OvercurrentResetTimer = 0;
			Mains_Contactor = OFF;
			ToggleMains = FALSE;
			INTCON  = 0b00000000;

			SineCounter++;
			if (SineCounter >=99)
				SineCounter = 0;
			MainsSineCounter++;
			if (MainsSineCounter >= 99)
			{
				MainsSineCounter = 0;
				MaxOutputCurrentP=MaxOutputCurrent;
				MaxOutputCurrent = 0;
				MaxOutputVoltsP = MaxOutputVolts;
				MaxOutputVolts = 0;
				MaxBatteryCurrentP = MaxBatteryCurrent;
				MaxBatteryCurrent = 0;
				MaxGeneratorVoltsP = MaxGeneratorVolts;
				MaxGeneratorVolts = 0;
				MaxBatteryVoltsP = MaxBatteryVolts;
				MaxBatteryVolts = 0;
				AverageGeneratorVoltsP = AverageGeneratorVolts>>5;
				AverageGeneratorVolts = 0;
				AverageOutputVoltsP = AverageOutputVolts>>3;
				AverageOutputVolts = 0;
				MaxMainsInputCurrentP=MaxMainsInputCurrent;
				MaxMainsInputCurrent = 0;

			}	
		if (Tripped)
		{
			Led1 	= ON;
			Led2	= ON;
			Led3	= ON;
			Led4	= ON;
			Led5	= ON;

		}
		else
		{
			Led1 	= OFF;
			Led2	= OFF;
			Led3	= OFF;
			Led4	= OFF;
			Led5	= OFF;
		}

		if ((System_On) && (!Tripped))
			IStat = I_STARTING;
								
		break;				


		case I_STARTING	:
		
			// Set up PWM control
			TRISDbits.TRISD4 = 1;	// Set up PWM TRIS bits as High Impedance 
			TRISDbits.TRISD5 = 1;   // inputs before setting ECCPCON register
			PR2 = 255;              // started at 125
			ECCP1CON = 0b10001100;
			ECCP1DEL = 10;
			T2CON = 0b00000100;
			//Wait for 1st complete PWM cycle
			while(!PIR1bits.TMR2IF);
			TRISDbits.TRISD4 = 0;
			TRISDbits.TRISD5 = 0;
		
			SoftStart++;
			OutputScale = SoftStart / 40;
			
			if (External_Mains_On)  
			{
				tmplong = (long)(GeneratorVolts-IntOffset)*(long)(OutputScale);
				Led3 = OFF;
				Led4 = ON;
				if (GenVoltsPhase == CROSSINGZERO)
					SineCounter = 0;
			}
			else
			{
				tmplong = (long)(Sine[SineCounter]-Offset)*(long)(OutputScale);
				Led3 = ON;
				Led4 = OFF;
			}
			
			
			ServoInput = (int)(tmplong/127);

			tempint = ServoInput + Offset;

			if (tempint > 1023)
				tempint = 1023;
			if (tempint < 1)
				tempint = 1;
				
			if (tempint && 0b0000000000000001)
				ECCP1CONbits.EDC1B0 = 1;
			else
				ECCP1CONbits.EDC1B0 = 0;
				
			if (tempint && 0b0000000000000010)
				ECCP1CONbits.EDC1B1 = 1;
			else
				ECCP1CONbits.EDC1B1 = 0;
				
			Output = (unsigned char)(tempint/4);
		
			if (Output > 255)
				Output = 255;
			if (Output < 1)
				Output = 1;
			
			ECCPR1L = Output;

			if (AverageSineCounter == 99)
			{
				SineCounter = 0;
				AverageSineCounter = 0;
				MaxOutputCurrentP=MaxOutputCurrent;
				MaxOutputCurrent = 0;
				MaxOutputVoltsP = MaxOutputVolts;
				MaxOutputVolts = 0;
				MaxBatteryCurrentP = MaxBatteryCurrent;
				MaxBatteryCurrent = 0;
				MaxGeneratorVoltsP = MaxGeneratorVolts;
				MaxGeneratorVolts = 0;
				MaxBatteryVoltsP = MaxBatteryVolts;
				MaxBatteryVolts = 0;
				AverageGeneratorVoltsP = AverageGeneratorVolts>>5;
				AverageGeneratorVolts = 0;
				AverageOutputVoltsP = AverageOutputVolts>>3;
				AverageOutputVolts = 0;
				MaxMainsInputCurrentP=MaxMainsInputCurrent;
				MaxMainsInputCurrent = 0;
			}
			else
			{
				AverageSineCounter++;
				SineCounter++;
			}
			if ((!System_On)||(Tripped)||(ToggleMains))
				IStat = I_SWITCHED_OFF;
			
			if ((SoftStart >= 5080)||(AverageOutputVoltsP >= SETPOINT))
			{
				Led1 = OFF;
				IStat = I_RUNNING;
			}
		
		break;
		
		case I_RUNNING :

			Led1 = ON;
			if (External_Mains_On)
			{
				ServoInput = (GeneratorVolts-IntOffset-GENERATOR_VOLTS_OFFSET);
				ServoInput = ServoInput;
				Led3 = OFF;
				Led4 = ON;
				DCOffset = EXTERNAL_DC_OFFSET; 
				MaxOutputScale = 200;
				if (GenVoltsPhase == CROSSINGZERO)
					SineCounter = 0;
			}
			else
			{
				ServoInput = (int)(Sine[SineCounter])-Offset;
				Led3 = ON;
				Led4 = OFF;
				DCOffset  = INTERNAL_DC_OFFSET;
				MaxOutputScale = 128;
				DeSyncCounter--;
				if (DeSyncCounter < 2)
				{
					DeSyncCounter = 1;
					DeSyncing = FALSE;
				}
			}
		
			if (ServoInput < 0)
			{
				tempint = ServoInput * -1;
				templong = (long)tempint * (long)OutputScale;
				if (templong > 65530)
					templong = 65530;
				ServoOutput = (unsigned int)templong;//tempint * OutputScale;
				tempint = ServoOutput/128;
				tempint = (-1 * tempint) + Offset - DCOffset;
			}
			else
			{
				templong = (long)ServoInput * (long)OutputScale;
				if (templong > 65530)
					templong = 65530;
				ServoOutput = (unsigned int)templong;
				tempint = ServoOutput/128;
				tempint = tempint + Offset - DCOffset;
			}	
			
			if (tempint > PWMMax)
				PWMMax = tempint;
				
			if (tempint >= 1023)
			{
				PWMLim = TRUE;
				tempint = 1023;
			}
				
			if (tempint < 1)
				tempint = 1;
				
			if (tempint && 0b0000000000000001)
				ECCP1CONbits.EDC1B0 = 1;
			else
				ECCP1CONbits.EDC1B0 = 0;
				
			if (tempint && 0b0000000000000010)
				ECCP1CONbits.EDC1B1 = 1;
			else
				ECCP1CONbits.EDC1B1 = 0;
				
			tempint = tempint / 4;
			
			Output = (unsigned char)tempint;
			
			ECCPR1L = Output;

			if (AverageSineCounter == 99)
			{
				Led2 = OFF;
				Led3 = OFF;
				Led4 = OFF;
				/********First Get Output Synchronised with External Mains if necessary************/
				if ((External_Mains_On) && (Mains_Contactor == OFF))
				{
					if (PWMLim)
					{
						ErrorGain--;
						PWMLim = FALSE;
					}
					else
					{
						if (AverageGeneratorVoltsP > AverageOutputVoltsP)
							ErrorGain++;
						else
							ErrorGain--;
					}
				}
				else
				{  
					/*******Scale here with slow integrator******************************************/
					/* First Check if Battery Charging Current is Too High							*/
					Error = SETPOINT - AverageOutputVoltsP;
					if (PWMLim)
					{
						ErrorGain = ErrorGain--;
						PWMLim = FALSE;
					}
					else
					{			
						if (Error > 0)
						{
							if ((Error > 45) && (!DeSyncing))
								ErrorGain = ErrorGain + 3;
							else
								ErrorGain = ErrorGain + 1;
						}
						else if (Error < 0) 	
						{
							if (Mains_Contactor == ON)	// This means batteries are charging
							{
								Led2 = ON;
								if (MaxBatteryVoltsP > MAX_BATTERY_VOLTS)
									ErrorGain++;
								else if (MaxBatteryCurrentP > MAX_CHARGING_CURRENT)
									ErrorGain = ErrorGain + 1;
								else
									ErrorGain = ErrorGain - 1;
							}
							else if (Error < -45)
								ErrorGain = ErrorGain - 3;
							else
								ErrorGain = ErrorGain - 1;
						}
					}
				}
				if (ErrorGain > MaxOutputScale)
					ErrorGain = MaxOutputScale;
				if (ErrorGain < 80)
					ErrorGain = 80;
	
				if (!External_Mains_On)
					SineCounter = 0;

				OutputScale = ErrorGain;
				AverageSineCounter = 0;
				MaxOutputCurrentP=MaxOutputCurrent;
				MaxOutputCurrent = 0;
				MaxOutputVoltsP = MaxOutputVolts;
				MaxOutputVolts = 0;
				MaxBatteryCurrentP = MaxBatteryCurrent;
				MaxBatteryCurrent = 0; 
				MaxGeneratorVoltsP = MaxGeneratorVolts;
				MaxGeneratorVolts = 0;
				MaxBatteryVoltsP = MaxBatteryVolts;
				MaxBatteryVolts = 0;
				AverageGeneratorVoltsP = AverageGeneratorVolts>>5;
				AverageGeneratorVolts = 0;
				AverageOutputVoltsP = AverageOutputVolts>>3;
				AverageOutputVolts = 0;
				MaxMainsInputCurrentP=MaxMainsInputCurrent;
				MaxMainsInputCurrent = 0;
				PWMMaxP = PWMMax;
				PWMMax = 0;
			}
			else
			{
				AverageSineCounter++;
				SineCounter++;
			}

		
				
		if ((!System_On)||(Tripped)||(ToggleMains))
		{
			IStat = I_SWITCHED_OFF;
			ToggleMainsFlag = TRUE;
		}	
		break;
	}
		
}

void GENBOOST1_Reset_TripCounter(void)
{
	PrimaryOvercurrentTimer++;
	OvercurrentResetTimer++;
	if (!Tripped)
		if (OvercurrentResetTimer > 20000)
		{
			PrimaryOvercurrentCounter = 0;
			OvercurrentResetTimer = 0;
		} 

}

void GENBOOST1_Check_Mains_In(void)
{
	GenReverseFeed = FALSE;
	switch(MainsInputStatus)
	{
		case MAINS_OFF :
			ContactorDelayTimer = MAINSCONTACTORONDELAY;
			if ((AverageGeneratorVoltsP > MIN_GENERATOR_ON_VOLTS)&&(!GenReverseFeed) && (Generator == ON))
				MainsOnDelayTimer--;

			if (MainsOnDelayTimer < 2)
				MainsInputStatus = SYNCHRONIZE_MAINS;
		break;
				
		case SYNCHRONIZE_MAINS :
			if ((AverageGeneratorVoltsP < MAX_GENERATOR_OFF_VOLTS)||(GenReverseFeed)|| (Generator == OFF))
			{
				MainsInputStatus = MAINS_OFF;
				MainsOnDelayTimer = MAINSONDELAYTIME;
			}
			else if (VoltagePhase < 500)
			{
				External_Mains_On = TRUE; //This causes the system PWM to follow the external mains input
				MainsInputStatus = CONNECT_MAINS_CONTACTOR;
			}
		break;

		case CONNECT_MAINS_CONTACTOR :
			if ((AverageGeneratorVoltsP < MAX_GENERATOR_OFF_VOLTS)||(GenReverseFeed)||(Generator == OFF))
				MainsInputStatus = DISCONNECT_MAINS_CONTACTOR;
			else if (AverageGeneratorVoltsP >= AverageOutputVoltsP)
					ContactorDelayTimer--;

			if (ContactorDelayTimer == 0)
			{
				Mains_Contactor = ON;
				PORTEbits.RE1 = 1;
				Led5 = ON;
				MainsInputStatus = MAINS_RUNNING;
			}
		break;
	
		case MAINS_RUNNING :
			if ((AverageGeneratorVoltsP < MAX_GENERATOR_OFF_VOLTS)||(GenReverseFeed)||(Generator == OFF))
				MainsInputStatus = DISCONNECT_MAINS_CONTACTOR;

		break;

		case DISCONNECT_MAINS_CONTACTOR :

				if (AverageGeneratorVoltsP < MAX_GENERATOR_OFF_VOLTS)
				{
					ToggleMains = TRUE;
					MainsInputStatus = MAINS_OFF;
					External_Mains_On = FALSE;
				}
				else if (GenVoltsPhase == CROSSINGZERO)
				{
					Mains_Contactor = OFF;
					PORTEbits.RE1 = 0;
					External_Mains_On = FALSE;
					MainsOnDelayTimer = MAINSONDELAYTIME;
					Led5 = OFF;
					DeSyncCounter = DESYNCTIME;
					DeSyncing = TRUE;
					SineCounter = 0;
					MainsInputStatus = MAINS_OFF;
				}
		break;
	}

}

void GENBOOST1_Check_System_Status(void)
{
	if (PORTDbits.RD7)
		System_On = FALSE;
	else
		System_On = TRUE;

		System_On = TRUE;
}


void GENBOOST1_Switch_Leds(void)
{
	if (Led1)
		PORTDbits.RD0 = 1;
	else
		PORTDbits.RD0 = 0;
	
	if (Led2)
		PORTDbits.RD1 = 1;
	else
		PORTDbits.RD1 = 0;
		
	if (Led3)
		PORTDbits.RD2 = 1;
	else
		PORTDbits.RD2 = 0;
	
	if (Led4)
		PORTDbits.RD3 = 1;
	else
		PORTDbits.RD3 = 0;

	if (Led5)
		PORTDbits.RD6 = 1;
	else
		PORTDbits.RD6 = 0;

	if ((IStat == I_RUNNING) ||(IStat == I_STARTING))
		PORTCbits.RC3 = 1;
	else
		PORTCbits.RC3 = 0;
}

UCHAR HexConvert(UCHAR convtxt)
{ 
		switch(convtxt)
		{
			case 0	:
				return('0');
			break;
		
			case 1 	:
				return('1');
			break;
	
			case 2	:
				return('2');
			break;
	
			case 3	:
				return('3');
			break;
	
			case 4 	:
				return('4');
			break;
	
			case 5	:
				return('5');
			break;

			case 6	:
				return('6');
			break;
	
			case 7 	:
				return('7');
			break;
	
			case 8	:
				return('8');
			break;
	
			case 9	:
				return('9');
			break;
	
			case 10 :
				return('A');
			break;
	
			case 11	:
				return('B');
			break;

			case 12 	:
				return('C');
			break;
	
			case 13	:
				return('D');
			break;
	
			case 14	:
				return('E');
			break;
	
			case 15 :
				return('F');
			break;
		}
}

void HexStringConv(void)
{
	TmpString[0] = HexConvert(VBunion.br[0]&0b00001111);
	TmpString[1] = HexConvert((VBunion.br[0]&0b11110000)>>4);
	TmpString[2] = HexConvert(VBunion.br[1]&0b00001111);
	TmpString[3] = HexConvert((VBunion.br[1]&0b11110000)>>4);
	ChrLength = 4;
}

void GENBOOST1_VBComms(void)
{
//	static UCHAR	LineNum=1;
	static UCHAR	CharNum=0;
	static char		c;
	
	switch (LineNum)
	{
		case 1 :
			if (FirstPass)
			{
				VBunion.lr = AverageOutputVoltsP;
				HexStringConv();
			}
		break;
				
		case 2 :
			if (FirstPass)
			{
				VBunion.lr = MaxOutputCurrentP;
				HexStringConv();
			}
		break;
				
		case 3 :
			if (FirstPass)
			{
				VBunion.lr = MaxGeneratorVoltsP;
				HexStringConv();
			}
		break;
				
		case 4 :
			if (FirstPass)
			{
				VBunion.lr = MaxBatteryVoltsP;
				HexStringConv();
			}
		break;
		
		case 5 :
			if (FirstPass)
			{
				VBunion.lr = MaxBatteryCurrentP;
				HexStringConv();
			}
		break;
		
		case 6 :
			if (FirstPass)
			{
				VBunion.br[1] = 0;
				VBunion.br[0] = Tripped;
				HexStringConv();
			}
		break;
		
		case 7 :
			if (FirstPass)
			{
				VBunion.br[1] = 0;
				VBunion.br[0] = Mains_Contactor;
				HexStringConv();
			}
			
		break;
		
		case 8 :
			if (FirstPass)
			{
				VBunion.br[1] = 0;
				VBunion.br[0] = System_On;
				HexStringConv();
			}
			
		break;
		
		case 9 :
			if (FirstPass)
			{
				VBunion.lr = SETPOINT;
				HexStringConv();
			}
		break;
		
		case 10 :
			if (FirstPass)
			{
				VBunion.br[0] = CodeVersionSub;
				VBunion.br[1] = CodeVersionMain;
				HexStringConv();
			}
		break;
		
		case 11	:
			if (FirstPass)
			{
				VBunion.lr = OutputScale;
				HexStringConv();
			}
			
		break;

		case 12	:
			if (FirstPass)
			{
				VBunion.lr = AverageGeneratorVoltsP;	
				HexStringConv();
			}
			
		break;
	
		case 13	:
			if (FirstPass)
			{
				VBunion.lr = PWMMaxP;	
				HexStringConv();
			}
			
		break;
	
		case 14 :
			if (FirstPass)
			{
				VBunion.lr = MaxMainsInputCurrentP;
				HexStringConv();
			}
		break;

		case 15 :
			if (FirstPass)
			{
				VBunion.lr = VoltagePhase;
				HexStringConv();
			}
		break;

		case 16 :
			if (FirstPass)
			{
				VBunion.lr = ToggleMainsFlag;
				HexStringConv();
			}
		break;
			

	}	
	
	FirstPass = FALSE;
	

	c = TmpString[CharNum];
	CharNum++;

	if (LineNum < 17)
	{
		if (CharNum >= ChrLength)
		{
			CharNum = 0;
			LineNum++;
			FirstPass = TRUE;
		}
	}
	else
	{
		c = 0x0C; // new page
		LineNum = 1;
		CharNum = 0;
		FirstPass = TRUE;
	}
		
	putcUSART(c);

}

void main (void)
{
	
	GENBOOST1_Initialise();

	//System Has just Woken up (been switched on) so precharge Capacitors then Switch on Battery Contactor

	PORTCbits.RC2 = 1;	//Switch on Precharge CCt
	for (icounter=1;icounter<500;icounter++)	//Now wait for a few seconds
	{
		WriteTimer0(0);
		while (ReadTimer0() < 65000);
	}

	PORTEbits.RE2 = 1;	//Switch on Battery Disconnect Contactor
	PORTCbits.RC2 = 0;	//Switch off Precharge CCT	

	WriteTimer0(1665);

	while (1<2)
	{	  	
  		
  		if (PrimaryOvercurrentTimer > 30000)
		{
			PrimaryOvercurrentTimer = 30001;
			INTCON  = 0b10000000;
			INTCON2 = 0b01000000;
			INTCON3 = 0b00001000;
		}
		timer.bt[0] = TMR0L;  // Copy Timer0 low byte into union
 		timer.bt[1] = TMR0H;  // Copy Timer0 high byte into union
  		Time = timer.lt;
		
		while (Time < 1539)
		{
			timer.bt[0] = TMR0L;  // Copy Timer0 low byte into union
 			timer.bt[1] = TMR0H;  // Copy Timer0 high byte into union
  			Time = timer.lt;
		}
		WriteTimer0(0);
	
  			
  		//	if ((Time > 1300) && (Channel == VMON))
  		//	{
  				
  		//		GENBOOST1_ReadOutputVolts();
  		//		Channel = NextADCChan;
  		//		SetChanADC(Channel<<3);
  				
		//	}
		
		INTCON = 0b00000000;  // Disable Interrupts		
		GENBOOST1_ReadGenVolts();
		SetChanADC(NextADCChan<<3);
					
		GENBOOST1_Run_Inverter();
		Time = ReadTimer0();
		Nop();
		GENBOOST1_ReadADC();
		GENBOOST1_ReadCAN();		
		GENBOOST1_Reset_TripCounter();
		GENBOOST1_Check_System_Status();
		GENBOOST1_Check_Mains_In();
		GENBOOST1_Switch_Leds();
		// Write to CAN at 200 Hz (Code itterates at 5KHz so once every 25 cycles
		CANOutputCounter++;
		if (CANOutputCounter > 24)
		{
			WriteCAN();
			CANOutputCounter = 0;
		}
		
		if (VBCommsCounter >  10)
		{
			GENBOOST1_VBComms();
			VBCommsCounter = 0;
		}
		else
			VBCommsCounter++;

		

	}		
}	



